@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
 from copy import deepcopy
-from functools import wraps
 import warnings
 import redis
+from funcy import memoize, decorator, identity
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 
 
+
 profile_defaults = {
     'ops': (),
     'local_get': False,
@@ -23,21 +24,20 @@ for key in profiles:
     profiles[key] = dict(profile_defaults, **profiles[key])
 
 
-# Support degradation on redis fail
+STRICT_STRINGIFY = getattr(settings, 'CACHEOPS_STRICT_STRINGIFY', False)
 DEGRADE_ON_FAILURE = getattr(settings, 'CACHEOPS_DEGRADE_ON_FAILURE', False)
 
-def handle_connection_failure(func):
-    if not DEGRADE_ON_FAILURE:
-        return func
 
-    @wraps(func)
-    def _inner(*args, **kwargs):
+# Support DEGRADE_ON_FAILURE
+if DEGRADE_ON_FAILURE:
+    @decorator
+    def handle_connection_failure(call):
         try:
-            return func(*args, **kwargs)
+            return call()
         except redis.ConnectionError as e:
             warnings.warn("The cacheops cache is unreachable! Error: %s" % e, RuntimeWarning)
-
-    return _inner
+else:
+    handle_connection_failure = identity
 
 class SafeRedis(redis.StrictRedis):
     get = handle_connection_failure(redis.StrictRedis.get)
@@ -52,8 +52,7 @@ except AttributeError:
 redis_client = (SafeRedis if DEGRADE_ON_FAILURE else redis.StrictRedis)(**redis_conf)
 
 
-model_profiles = {}
-
+@memoize
 def prepare_profiles():
     """
     Prepares a dict 'app.model' -> profile, for use in model_profile()
@@ -61,6 +60,7 @@ def prepare_profiles():
     if hasattr(settings, 'CACHEOPS_PROFILES'):
         profiles.update(settings.CACHEOPS_PROFILES)
 
+    model_profiles = {}
     ops = getattr(settings, 'CACHEOPS', {})
     for app_model, profile in ops.items():
         profile_name, timeout = profile[:2]
@@ -75,16 +75,14 @@ def prepare_profiles():
         mp['timeout'] = timeout
         mp['ops'] = set(mp['ops'])
 
-    if not model_profiles and not settings.DEBUG:
-        raise ImproperlyConfigured('You must specify non-empty CACHEOPS setting to use cacheops')
-
+    return model_profiles
 
+@memoize
 def model_profile(model):
     """
     Returns cacheops profile for a model
     """
-    if not model_profiles:
-        prepare_profiles()
+    model_profiles = prepare_profiles()
 
     app = model._meta.app_label
     # module_name is fallback for Django 1.5-
